<?php
// $Id $

/**
 * @file
 * project_issue integration code.
 */

/**
 * Note: this code is mostly taken and adapted from issue.inc and taxonomy.module
 * This is a somewhat ugly approach to stop a gap on d.o before upgrading to 6
 * (and hopefully to Views...).
 */

/**
 * Display the list of issues matching given terms.
 */
function comment_alter_taxonomy_project_issue_page($project_name = NULL, $str_tids, $depth = 0, $op = 'page') {
  $uris = NULL;
  $projects = array(0 => t('<all>')) + project_projects_select_options($uris);
  $states = array(implode(',', array_keys(project_issue_state())) => t('<all>')) + project_issue_state();
  $priorities = array(0 => t('<all>')) + project_issue_priority();

  $project = project_project_retrieve($project_name);
  if (isset($project->nid) && $project_nid = $project->nid) {
    if (!node_access('view', $project)) {
      drupal_access_denied();
    }
    if (module_exists('project_release')) {
      $releases = project_release_get_releases($project, 0);
    }
    else {
      $releases = array();
    }
  }
  else {
    $project_nid = NULL;
    $releases = array();
  }

  $terms = taxonomy_terms_parse_string($str_tids);
  if ($terms['operator'] != 'and' && $terms['operator'] != 'or') {
    drupal_not_found();
  }

  if ($terms['tids']) {
    $placeholders = implode(',', array_fill(0, count($terms['tids']), '%d'));
    $result = db_query(db_rewrite_sql('SELECT t.tid, t.name FROM {term_data} t WHERE t.tid IN ('. $placeholders .')', 't', 'tid'), $terms['tids']);
    $tids = array(); // we rebuild the $tids-array so it only contains terms the user has access to.
    $names = array();
    while ($term = db_fetch_object($result)) {
      $tids[] = $term->tid;
      $names[] = $term->name;
    }

    if ($names) {
      if ($project_nid) {
        $title = t("Issues for %project tagged with %title", array('%project' => $project->title, '%title' => implode(', ', $names)));
      }
      else {
        $title = t("Project issues tagged with %title", array('%title' => implode(', ', $names)));
      }
      drupal_set_title($title);

      switch ($op) {
        case 'page':

          $issues_per_page = variable_get('project_issues_per_page', PROJECT_ISSUES_PER_PAGE);

          $header = array();
          if (!$project->nid) {
            $header[] = t('Project');
          }
          $header[] = t('Summary');
          $header[] = t('Status');
          $header[] = t('Priority');
          $header[] = t('Category');
          if (!empty($releases)) {
            $header[] = t('Version');
          }
          $header[] = t('Last updated');
          $header[] = t('Assigned to');

          $result = comment_alter_taxonomy_project_select_nodes($tids, $terms['operator'], $project_nid, $depth, TRUE);

          if ($project_nid) {
            drupal_add_feed(url('project/issues/'. $project_nid .'/term/'. $str_tids .'/'. $depth .'/feed'), 'RSS - '. $title);
          }
          else {
            drupal_add_feed(url('project/issues-term/'. $str_tids .'/'. $depth .'/feed'), 'RSS - '. $title);
          }

          if (project_use_taxonomy()) {
            $flat_projects = array();
            foreach ($projects as $tmp) {
              if (is_array($tmp)) {
                $flat_projects += $tmp;
              }
            }
            $projects = $flat_projects;
          }

          $rows = array();

          while ($node = db_fetch_object($result)) {
            $node = node_load($node->nid);
            $row = array();
            $class = "state-$node->sid";
            if (!$project->nid) {
              $row[] = l($projects[$node->pid], "project/issues/$node->pid");
            }
            if (strlen($node->title) > 50) {
              $title = l(drupal_substr($node->title, 0, 50), "node/$node->nid", array('title' => $node->title));
            }
            else {
              $title = l($node->title, "node/$node->nid");
            }
            $row[] = $title . theme('mark', node_mark($node->nid, $node->changed));
            $row[] = $states[$node->sid];
            $row[] = $priorities[$node->priority];
            $row[] = project_issue_category($node->category, 0);
            if (count($releases)) {
              $row[] = $releases[$node->rid];
            }
            $row[] = array('data' => format_interval(time() - $node->changed, 2), 'align' => 'right');
            $row[] = ($node->assigned) ? theme('username', user_load(array('uid' => $node->assigned))) : '';

            $row = array('data' => $row, 'class' => $class);

            $rows[] = $row;
          }

          if (!count($rows)) {
            $rows[] = array(array('data' => t('No issues found.'), 'colspan'=> 8));
          }

          if ($pager = theme('pager', NULL, $issues_per_page, 0)) {
            $rows[] = array(array('data' => $pager, 'colspan' => count($header)));
          }

          drupal_add_css(drupal_get_path('module', 'project_issue') .'/project_issue.css');

          $output = '<div class="project-issue">';
          $output .= theme('table', $header, $rows);
          $output .= '</div>';

          return $output;
          break;

        case 'feed':
          $term = taxonomy_get_term($tids[0]);
          if ($project_nid) {
            $channel['link'] = url('project/issues/'. $project_nid .'/term/'. $str_tids .'/'. $depth, NULL, NULL, TRUE);
          }
          else {
            $channel['link'] = url('project/issues-term/'. $str_tids .'/'. $depth, NULL, NULL, TRUE);
          }
          $channel['title'] = variable_get('site_name', 'Drupal') .' - '. $title;
          $channel['description'] = $title;

          $result = comment_alter_taxonomy_project_select_nodes($tids, $terms['operator'], $project_nid, $depth, FALSE);
          node_feed($result, $channel);
          break;
        default:
          drupal_not_found();
      }
    }
    else {
      drupal_not_found();
    }
  }
}

/**
 * Finds all nodes that match selected taxonomy conditions.
 *
 * This is taken from taxonomy_select_nodes(), and thus, as ugly as the original
 * function is. Hmm... Drupal 7 query builder...
 *
 * @param $tids
 *   An array of term IDs to match.
 * @param $operator
 *   How to interpret multiple IDs in the array. Can be "or" or "and".
 * @param $project_nid
 *   Optional project nid to restrict issues with.
 * @param $depth
 *   How many levels deep to traverse the taxonomy tree. Can be a nonnegative
 *   integer or "all".
 * @param $pager
 *   Whether the nodes are to be used with a pager (the case on most Drupal
 *   pages) or not (in an XML feed, for example).
 * @param $order
 *   The order clause for the query that retrieve the nodes.
 * @return
 *   A resource identifier pointing to the query results.
 */
function comment_alter_taxonomy_project_select_nodes($tids = array(), $operator = 'or', $project_nid = NULL, $depth = 0, $pager = TRUE, $order = 'ORDER BY n.sticky DESC, n.changed DESC') {
  if (count($tids) > 0) {
    // For each term ID, generate an array of descendant term IDs to the right depth.
    $descendant_tids = array();
    if ($depth === 'all') {
      $depth = NULL;
    }
    foreach ($tids as $index => $tid) {
      $term = taxonomy_get_term($tid);
      $tree = taxonomy_get_tree($term->vid, $tid, -1, $depth);
      $descendant_tids[] = array_merge(array($tid), array_map('_taxonomy_get_tid_from_term', $tree));
    }

    $joins = array();
    $args = array();
    $wheres = array();

    if (isset($project_nid)) {
      $joins[] = "INNER JOIN {project_issues} pi ON pi.nid = n.nid";
      $wheres[] = "pi.pid = %d";
      $args[] = $project_nid;
    }

    $joins[] = 'INNER JOIN {term_node} tn ON n.nid = tn.nid';
    if ($operator == 'or') {
      $tids = call_user_func_array('array_merge', $descendant_tids);
      $placeholders = implode(',', array_fill(0, count($tids), '%d'));
      $wheres[] = "tn.tid IN (". $placeholders .")";
      $args = array_merge($args, $tids);
    }
    else {
      foreach ($descendant_tids as $index => $tids) {
        $joins[] = 'INNER JOIN {term_node} tn'. $index .' ON n.nid = tn'. $index .'.nid';
        $placeholders = implode(',', array_fill(0, count($tids), '%d'));
        $wheres[] = 'tn'. $index .'.tid IN ('. $placeholders .')';
        $args = array_merge($args, $tids);
      }
    }

    $wheres[] = 'n.status = 1';
    $wheres[] = "n.type = 'project_issue'";

    $sql = 'SELECT DISTINCT(n.nid) FROM {node} n ' . implode(' ', $joins) . ' WHERE ' . implode(' AND ', $wheres) . ' ' . $order;
    $sql_count = 'SELECT COUNT(DISTINCT(n.nid)) FROM {node} n ' . implode(' ', $joins) . ' WHERE ' . implode(' AND ', $wheres);

    $sql = db_rewrite_sql($sql);
    $sql_count = db_rewrite_sql($sql_count);

    if ($pager) {
      $result = pager_query($sql, variable_get('project_issues_per_page', PROJECT_ISSUES_PER_PAGE), 0, $sql_count, $args);
    }
    else {
      $result = db_query_range($sql, $args, 0, variable_get('feed_default_items', 10));
    }
  }

  return $result;
}
